#include <values.h>
#include "cluster_set.h"
#include "cluster.h"
#include "types.h"
#include "classe.h"
#include "conversion.h"

#define Nb_cluster     SELF(nb_cluster)
#define Clusters       SELF(clusters)
#define Equiv          SELF(equiv)

static void make_clusters_priv(cluster_set,
			       cluster,
			       int);
static void compute_means_priv(cluster_set);

struct cluster_set
{
  int     nb_cluster;
  int*    equiv;
  cluster clusters;
};
cluster_set FAIRE_cluster_set(image img,
			      partition init_part,
			      int nb_cluster,
			      int space)
{
  int i,j;
  int * pixel;
  int C[3];
  int clus;
  cluster ptr,ptr_aux;
  PFconversion *rgb_to_c1c2c3 = changer_espace(space);
  INSTANCIER(cluster_set);

  ptr = (cluster)malloc(nb_cluster*sizeof(struct cluster));
  Nb_cluster = 0;

  ptr_aux = ptr;
  for(i=0;i<nb_cluster;i++)
    {
      ptr_aux->mom0 = 0.0;      
      for(j=0;j<3;j++)
	{	
	  ptr_aux->mom1[j] = 0.0;
	  ptr_aux->mom2[j] = 0.0;
	}
      ptr_aux++;
    }
  image_debut(img);
  do
    {
      pixel = image_lire_pixel(img);

      clus = partition_give_cluster_nb(init_part,pixel);
      for(i=0;i<3;i++)
	C[i] = rgb_to_c1c2c3[i](pixel[0],pixel[1],pixel[2]);
      
      if(cluster_is_empty(ptr+clus))
	Nb_cluster++;
      cluster_add_color_to_cluster(ptr+clus,C);
    }
  while(image_pixel_suivant(img));
  
  if(Nb_cluster == nb_cluster)
    {
      Clusters = ptr;
      compute_means_priv(self);

      return self;
    }
  
  make_clusters_priv(self,ptr,nb_cluster);
  free(ptr);
  
  return self;
}

void DEFAIRE_cluster_set(cluster_set self)
{
  free(Clusters);
  free(self);
}

int cluster_set_give_cardinal(cluster_set self)
{
  return Nb_cluster;
}
/* On ne traite pas les intervalles qui n'ont apres le merges a priori */
/* plus de signification */
void cluster_set_merge_cluster(cluster_set self,int clus1,int clus2)
{
  double alpha,mom0c1,mom0c2;
  cluster c1,c2;
  int i;

  c1 = Clusters+clus1;
  c2 = Clusters+clus2;

  c2->exist = faux;
  c2->canonic = clus1;

  mom0c1 = c1->mom0;
  mom0c2 = c2->mom0;
  alpha  = (mom0c1*mom0c2)/(mom0c1+mom0c2);
  c1->sqe += c2->sqe + alpha*vecteur_dist(c1->mean,c2->mean);

  c1->mom0 += mom0c2;

  for(i=0;i<3;i++)
    {
      c1->mom1[i] += c2->mom1[i];
      c1->mom2[i] += c2->mom2[i];
      c1->mean[i] = c1->mom1[i]/c1->mom0;
    }
}
/* A n'utiliser qu'en debut de programme quand les intervalles sont */
/* encore valides */
bouleen cluster_set_intersect(cluster_set self,int clus1,int clus2)
{
  int i;
  intervalle *inter1,*inter2;

  inter1 = Clusters[clus1].inter;
  inter2 = Clusters[clus2].inter;

  i=0;
  while(i<3)
    {
      if((inter1[i].borne_sup == inter2[i].borne_inf)||
	 (inter1[i].borne_inf == inter2[i].borne_sup)
	 )
	return vrai;
      i++;
    }
  return faux;
}
double cluster_set_get_dist(cluster_set self,int clus1,int clus2)
{
  cluster c1,c2;
  double alpha,mom0c1,mom0c2;
  double dist;

  c1 = Clusters+ clus1;
  c2 = Clusters+ clus2;

  if(!c1->exist || !c2->exist)
    return DBL_MAX;

  mom0c1 = c1->mom0;
  mom0c2 = c2->mom0;
  alpha  = (mom0c1*mom0c2)/(mom0c1+mom0c2);
  dist   = alpha*vecteur_dist(c1->mean,c2->mean);

  return dist;
}
int cluster_set_give_canonic_cluster(cluster_set self,int clus)
{
  return Clusters[Equiv[clus]].canonic;
}
void cluster_set_set_canonic(cluster_set self)
{
  int i,ind;
  
  for(i=Nb_cluster-1;i>=0;i--)
    {
      if(Clusters[i].exist)
	Clusters[i].canonic = i;
      else
	{
	  ind = Clusters[i].canonic;
	  Clusters[i].canonic = Clusters[ind].canonic;
	}
    }
}
void cluster_set_give_canonic_clusters(cluster_set self,
				       int* means,
				       int* equiv)
{
  int i,j,ind;
  cluster ptr;
  
  ind = 0;
  ptr = Clusters;
  for(i=0;i<Nb_cluster;i++)
    {
      if(ptr->exist)
	{
	  for(j=0;j<3;j++)
	    {
	      *means = (int)ptr->mean[j];
	      means++;
	    }
	  *equiv = ind;
	  ind++;
	}
      equiv++;
      ptr++;
    }
}

static void make_clusters_priv(cluster_set self,
			       cluster ptr,
			       int nb_set)
{
  int ind;
  int i;

  Clusters = (cluster)malloc(Nb_cluster*sizeof(struct cluster));
  Equiv    = (int*)  malloc(nb_set*sizeof(int));

  ind =0;
  for(i=0;i<nb_set;i++)
    {
      if(!cluster_is_empty(ptr))
	{
	  cluster_copy(ptr,Clusters+ind);
	  Equiv[i] = ind;
	  ind++;
	}
      ptr++;
    }
}
static void compute_means_priv(cluster_set self)
{
  int i;

  for(i=0;i<Nb_cluster;i++)
    cluster_compute_mean_sqe(Clusters+i);
}
